"""Remove foreign key constraints on users.

Reason:
Foreign key constraints prevent users from getting deleted. Alternatively, we
can implement a soft delete of users in the future but for now, we'll need to
drop the key constraints.

Revision ID: 6c59b0b044f5
Revises: fd54ddf73521

"""
from alembic import op
import sqlalchemy as sa

import rasax.community.database.schema_migrations.alembic.utils as migration_utils


# revision identifiers, used by Alembic.
revision = "6c59b0b044f5"
down_revision = "fd54ddf73521"
branch_labels = None
depends_on = None


def delete_model_foreign_key(table, referred_table, constrained_column) -> None:
    fk_name = migration_utils.get_foreign_key(
        table, referred_table=referred_table, constrained_column=constrained_column
    )
    if fk_name is None:
        return

    if migration_utils.get_column(table, constrained_column) is not None:
        with op.batch_alter_table(table) as batch_op:  # type: op.BatchOperations
            batch_op.drop_constraint(fk_name)


def upgrade():

    delete_model_foreign_key("response", "rasa_x_user", "annotator_id")
    delete_model_foreign_key("git_repository", "rasa_x_user", "first_annotator_id")
    delete_model_foreign_key("nlu_training_data", "rasa_x_user", "annotator_id")
    delete_model_foreign_key("story", "rasa_x_user", "user")


def downgrade():
    if migration_utils.get_column("response", "annotator_id") is not None:
        with op.batch_alter_table("response") as batch_op:
            batch_op.create_foreign_key(
                "template_annotator_id_fkey",
                "rasa_x_user",
                ["annotator_id"],
                ["username"],
            )

    if migration_utils.get_column("git_repository", "first_annotator_id") is not None:
        with op.batch_alter_table("git_repository") as batch_op:
            batch_op.create_foreign_key(
                "fk_git_annotator_username",
                "rasa_x_user",
                ["first_annotator_id"],
                ["username"],
            )

    if migration_utils.get_column("nlu_training_data", "annotator_id") is not None:
        with op.batch_alter_table("nlu_training_data") as batch_op:
            batch_op.create_foreign_key(
                "nlu_training_data_annotator_id_fkey",
                "rasa_x_user",
                ["annotator_id"],
                ["username"],
            )

    if migration_utils.get_column("story", "user") is not None:
        with op.batch_alter_table("story") as batch_op:
            batch_op.create_foreign_key(
                "story_user_fkey", "rasa_x_user", ["user"], ["username"]
            )
